#include <QDial>
#include <QGridLayout>
#include <QLabel>
#include <QFrame>

#include "QOscilloscope.h"

QOscilloscope::QOscilloscope()
{
    oscope = new QOscope();

    QLabel *label_timescale = new QLabel("Time base");
    label_timescale->setAlignment( Qt::AlignCenter );
    label_timescale->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    timescale = new QDial();
    timescale->setRange(0,11);
    timescale->setPageStep(1);
    timescale->setValue(2);
    timescale->setNotchesVisible(true);
    timescale->setWrapping(false);

    QLabel *label_offset = new QLabel("X position");
    label_offset->setAlignment( Qt::AlignCenter );
    label_offset->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    offset = new QDial();
    offset->setRange(1,15);
    offset->setPageStep(1);
    offset->setValue(8);
    offset->setNotchesVisible(true);
    offset->setWrapping(false);

    QLabel *label_sync = new QLabel("Trigger");
    label_sync->setAlignment( Qt::AlignCenter );
    label_sync->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    sync = new QDial();
    sync->setRange(-40,40);
    sync->setPageStep(10);
    sync->setValue(0);
    sync->setNotchesVisible(true);
    sync->setWrapping(false);

    status = new QLabel("");
    status->setAlignment( Qt::AlignLeft );
    status->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    connect(timescale,  SIGNAL(valueChanged(int)),    oscope, SLOT(setTimescale(int)));
    connect(offset,     SIGNAL(valueChanged(int)),    oscope, SLOT(setOffset(int)));
    connect(sync,       SIGNAL(valueChanged(int)),    oscope, SLOT(setSync(int)));
    connect(oscope,     SIGNAL(haveMessage(QString)), this,   SLOT(setStatus(QString)));

    oscope->initHardware();

    QGridLayout *layout = new QGridLayout;
    layout->addWidget( oscope, 0, 0, 1, 3, Qt::AlignTop );
    layout->addWidget( label_offset, 1, 0, Qt::AlignTop );
    layout->addWidget( offset, 2, 0 );
    layout->addWidget( label_timescale, 1, 1, Qt::AlignTop );
    layout->addWidget( timescale, 2, 1 );
    layout->addWidget( label_sync, 1, 2, Qt::AlignTop );
    layout->addWidget( sync, 2, 2 );
    layout->addWidget( status, 3, 0, 1, 3, Qt::AlignTop );
    
    layout->setSpacing( 2 );
    layout->setMargin( 2 );
    
    setLayout( layout );
    
    setMaximumWidth( 270 );
}

QOscilloscope::~QOscilloscope()
{
}

void QOscilloscope::setStatus( QString s )
{
    status->setText( s );
}

#include "QOscilloscope.moc"
